package com.itron.i9bledemo;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Toast;

import com.company.i9sslclient.bluetooth.DeviceInfo;
import com.company.i9sslclient.bt.BluetoothController;
import com.company.i9sslclient.lib.Logger;
import com.company.i9sslclient.listener.DeviceClick;

import java.util.ArrayList;
import java.util.List;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

public class ScanActivity extends AppCompatActivity  {

    private SwipeRefreshLayout mRefreshLayout;
    private RecyclerView mRecyclerView;

    private LinearLayoutManager mLinearLayoutManager;


    DeviceAdapter mPeotryAdapter;
    private List<DeviceInfo> deviceInfoList;
    Logger logger  = Logger.getInstance(ScanActivity.class);

    private BluetoothAdapter mBluetoothAdapter;
    private Handler mHandler;
    private boolean mScanning;
    private static final long SCAN_PERIOD = 10000;
    private List<String> devices;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scan);

        Toolbar toolbar = findViewById(R.id.toolbar);

        setSupportActionBar(toolbar);
        //设置是否有NvagitionIcon（返回图标）
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);



        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onBackPressed();
            }
        });

        // Use this check to determine whether BLE is supported on the device.  Then you can
        // selectively disable BLE-related features.
        if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
            Toast.makeText(this, R.string.ble_not_supported, Toast.LENGTH_SHORT).show();
            finish();
        }

        // Initializes a Bluetooth adapter.  For API level 18 and above, get a reference to
        // BluetoothAdapter through BluetoothManager.
        final BluetoothManager bluetoothManager =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();

        // Checks if Bluetooth is supported on the device.
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, R.string.error_bluetooth_not_supported, Toast.LENGTH_SHORT).show();
            finish();
            return;
        }
        mHandler = new Handler();





        mLinearLayoutManager = new LinearLayoutManager(this);



        mRefreshLayout = (SwipeRefreshLayout)findViewById(R.id.layout_swipe_refresh);




        deviceInfoList = new ArrayList<DeviceInfo>();
        mRecyclerView = (RecyclerView)findViewById(R.id.recyclerView);
        mRecyclerView.setLayoutManager(mLinearLayoutManager);
        mPeotryAdapter= new DeviceAdapter(this,deviceInfoList,new BtDeviceClick());
        mRecyclerView.setAdapter(mPeotryAdapter);
        devices =  new ArrayList<>();


        scanLeDevice(true);
    }


    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            /*
            case REQUEST_CONNECT_DEVICE_SECURE:
                // When DeviceListActivity returns with a device to connect
                if (resultCode == Activity.RESULT_OK) {
                    connectDevice(data, true);
                }
                break;
            case REQUEST_CONNECT_DEVICE_INSECURE:
                // When DeviceListActivity returns with a device to connect
                if (resultCode == Activity.RESULT_OK) {
                    connectDevice(data, false);
                }
                break;
                */
            case BluetoothController.REQUEST_ENABLE_BT:
                // When the request to enable Bluetooth returns
                if (resultCode == Activity.RESULT_OK) {
                    // Bluetooth is now enabled, so set up a chat session
                } else {
                    // User did not enable Bluetooth or an error occurred
                    Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                    finish();
                }
                break;

            case 0x20:

                mPeotryAdapter.setClicked(false);

                break;
        }
    }
    @Override
    public void onStart() {
        super.onStart();

        // If BT is not on, request that it be enabled.
        // setupChat() will then be called during onActivityResult
        if (!mBluetoothAdapter.enable()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, BluetoothController.REQUEST_ENABLE_BT);
            // Otherwise, setup the chat session
        }
    }




    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (isScanning()){
            scanLeDevice(false);
        }
    }

    public boolean isScanning(){
        return mScanning;
    }

    public void scanLeDevice(final boolean enable) {
        if (enable) {
            // Stops scanning after a pre-defined scan period.
            mHandler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    mScanning = false;
                    mBluetoothAdapter.stopLeScan(mLeScanCallback);
//                    invalidateOptionsMenu();
                }
            }, SCAN_PERIOD);

            mScanning = true;
            mBluetoothAdapter.startLeScan(mLeScanCallback);
        } else {
            mScanning = false;
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
        }
    }

    private BluetoothAdapter.LeScanCallback mLeScanCallback =
            new BluetoothAdapter.LeScanCallback() {

                @Override
                public void onLeScan(final BluetoothDevice device, int rssi, byte[] scanRecord) {
//                            mLeDeviceListAdapter.addDevice(device);
//                            mLeDeviceListAdapter.notifyDataSetChanged();
                    int size = devices.size();
                    if (size > 0){
                        boolean tmp = false;

                        for (int i=0;i<size;i++){
                            if (device.getAddress().equalsIgnoreCase(devices.get(i))){
                                tmp = true;
                                break;
                            }
                        }

                        if (!tmp){
                            if (device.getName() != null) {
                                devices.add(device.getAddress());
                                deviceInfoList.add(new DeviceInfo(device.getName(), device.getAddress()));
                                mPeotryAdapter.notifyDataSetChanged();
                            }

                        }

                    }else {
                        if (device.getName() != null) {
                            devices.add(device.getAddress());
                            deviceInfoList.add(new DeviceInfo(device.getName(), device.getAddress()));
                            mPeotryAdapter.notifyDataSetChanged();
                        }
                    }
                }
            };

    private final class BtDeviceClick implements DeviceClick {

        @Override
        public void onClick(final int position) {
            logger.error("position: "+position);

            logger.error("deviceInfoList.get(position).getIdentifier(): "+deviceInfoList.get(position).getIdentifier());

            int type = ScanActivity.this.getIntent().getIntExtra("TYPE",0);

            if (type == 0){
                Intent  intent = new Intent(ScanActivity.this, I9BleControlActivity.class);
                intent.putExtra("mac", deviceInfoList.get(position).getIdentifier());
                ScanActivity.this.startActivityForResult(intent,0x20);
            }else if (type == 1){
                Intent  intent = new Intent(ScanActivity.this, M6PlusNewControlActivity.class);
                intent.putExtra("mac", deviceInfoList.get(position).getIdentifier());
                ScanActivity.this.startActivity(intent);
            }else if (type ==2){
                Intent  intent = new Intent(ScanActivity.this, I9OnlenControlActivity.class);
                intent.putExtra("mac", deviceInfoList.get(position).getIdentifier());
                ScanActivity.this.startActivity(intent);
            }



        }
    }
}
