//
//  Network.swift
//  SwiftDemo
//
//  Created by itron on 2023/2/1.
//

import Foundation
import UIKit
import Alamofire
import SwiftyJSON

class Network {
    
    private var requestType: HTTPMethod = .get
    private var url: String?
    private var params: [String: Any]?
    private var httpRequest: Request?
}

extension Network {
    func url(_ url: String?) -> Self {
        self.url = url ?? ""
        return self
    }
    
    func requestType(_ type: HTTPMethod) -> Self {
        self.requestType = type
        return self
    }
    
    func params(_ params: [String: Any]?) -> Self {
        self.params = params
        return self
    }
}


extension Network {
    class func Request(_ method: HTTPMethod,
                       _ url: String,
                       _ params: [String: Any]?,
                       success: @escaping ((JSON) -> Void),
                       fail: @escaping ((String) ->Void)) {
     
        var requestHeaders = HTTPHeaders()
        requestHeaders.add(name: "Content-Type", value: "application/x-www-form-urlencoded")
        

        AF.sessionConfiguration.timeoutIntervalForRequest = 30
        AF.request(url,
                   method: method,
                   parameters: params,
                   encoding: URLEncoding.default,
                   headers: requestHeaders,
                   interceptor: nil,
                   requestModifier: nil)
        .validate()
        .responseJSON { (response) in
            guard let _ = response.value else {
                fail(response.error?.localizedDescription ?? "error")
                return
            }
            switch response.result {
            case .success(let dataJson):
                let resJson = JSON(dataJson);
                switch resJson["code"] {
                case "200",200:
                    success(resJson["data"]);
                    break;
                default:
                    fail(resJson["msg"].string!);
                    break;
                }
                break;
            case let .failure(error):
                fail(error.localizedDescription)
            }
        }
    }
}


extension Network {
    class func Download(_ method: HTTPMethod,
                       _ url: String,
                       _ params: [String: Any]?,
                       success: @escaping ((Data) -> Void),
                       fail: @escaping ((String) ->Void)) {
     
        var requestHeaders = HTTPHeaders()
        requestHeaders.add(name: "Content-Type", value: "application/x-www-form-urlencoded")
        

        AF.sessionConfiguration.timeoutIntervalForRequest = 30
        AF.download(url,parameters: params,encoding: URLEncoding.default,headers: requestHeaders).responseData { response in
            switch response.result{
            case .success(let data):
                success(data)
                break
            case let .failure(error):
                fail(error.localizedDescription)
            }
        }
    }
}
