//
//  ItronBaseClass.h
//  ItronLibrary
//
//  Created by 攀杨 on 2018/8/6.
//  Copyright © 2018年 攀杨. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "CompanyDelegate.h"
#import "CompanyClass.h"
#import "CompanyToolClass.h"

@interface CompanyBluetoothBaseClass : NSObject

+ (CompanyBluetoothBaseClass *)getInstance;
@property (nonatomic,weak)id<CompanyDelegate>delegate;
@property (nonatomic,assign)BOOL debug;



@property (nonatomic,assign)int deviceType;

#pragma Bluetooth

/**
 bluetooth connect state
 */
- (BOOL)isConnected;
/**
 search bluetooth

 @param delegate delegate
 @param timeout timeout
 */
- (void)searchDevices:(id<CompanyDelegate>)delegate timeout:(int)timeout;

/**
 stop search bluetooth
 */
- (void)stopSearching;

/**
 connect bluetooth by UUID

 @param UUIDString UUID of peripheral
 @param cb delegate
 @param timeout timeout
 */
- (void)openDevice:(NSString *)UUIDString cbDelegate:(id<CompanyDelegate>)cb timeout:(int)timeout;

/**
 connect bluetooth by peripheral

 @param peripheral peripheral
 @param cb delegate
 @param timeout timeout
 */
- (void)openDevice:(CBPeripheral *)peripheral delegate:(id<CompanyDelegate>)cb timeout:(int)timeout;

/**
 disconnect bluetooth
 */
-(void)closeDevice;
#pragma mark sendCommand
/**
 get device information
 */
- (void)getTerminalInfo;

/**
 Set terminal automatic shutdown time function

 @param time second
 */
- (void)setAutomaticShutdown:(int)time;


/**
 Update terminal time function

 @param datetime yyyyMMddHHmmss
 */
- (void)setTerminalDateTime:(NSString *)datetime;

/**
 get terminal time
 */
- (void)getTerminalDateTime;


/**
Download terminal communication key

 @param publickey RSA public key
 */
- (void)downloadRSApublicKey:(NSString *)publickey;


/**
 Set Card Acceptor Identification Code and Terminal Identification

 @param acceptor Acceptor Identification Cod
 @param terminal Terminal Identification
 */
-(void)setAcceptorTerminalIdentification:(NSString*)acceptor Terminal:(NSString *)terminal;


/**
 return Acceptor Identification Code
 */
- (void)getAcceptorIdentification;


/**
 return Terminal Identification
 */
- (void)getTerminalIdentification;

/**
 Stop current operation
 */
- (void)stopTrade;

///**PINEntry be removed. Use terminalEncryptionPIN instead
// pin entry
// @param pin pin
// */
//- (void)PINEntry:(NSString *)pin;

/**
 terminalEncryptionPIN
 
 @param pinData PIN clear text data
 @param entype  1:DUKPT ; 2:MK/SK
 @param index  index
 @param pinBlock PINBlock Format  (0, 1, 3, 4:MK/SK AES only)
 */
- (void)terminalEncryptionPIN:(NSString *)pinData entype:(int)entype index:(int)index pinBlock:(int)pinBlock;

- (void)updateBtPara:(NSString *)btNameStr;
- (void)updateTerminalPara:(NSString *)btNameStr;

/**
 requestDataEncrypt
 
 @param encryptData encryptData
 @param entype  1:DUKPT ; 2:MK/SK
 @param index  index
 @param random random
 @param dukptMode dukptMode 0 automatic addition of 1 , 1 dupkt  No automatic addition of 1
 @param operationMode operationMode 0 ECB , 1 CBC (IV data is required)
 @param ivData ivData
 */
- (void)requestDataEncrypt:(NSString *)encryptData entype:(int)entype index:(int)index dukptMode:(int)dukptMode random:(NSString *)random operationMode:(int)operationMode ivData:(NSString *)ivData;

/**
 requestDataDecrypt
 
 @param decryptData decryptData
 @param entype  1:DUKPT ; 2:MK/SK
 @param index  index
 @param random random
 @param dukptMode dukptMode 0 automatic addition of 1 , 1 dupkt  No automatic addition of 1
 @param operationMode operationMode 0 ECB , 1 CBC (IV data is required)
 @param ivData ivData
 */
- (void)requestDataDecrypt:(NSString *)decryptData entype:(int)entype index:(int)index dukptMode:(int)dukptMode random:(NSString *)random operationMode:(int)operationMode ivData:(NSString *)ivData;

/**
 getSecretKeyInfo
 
 @param keySystem  1:DUKPT ; 2:MK/SK
 @param entype    0：TDES ,    1：AES
 @param index  index
 @param keyType  0x01  PIN Key；0x02   Data Key；0x03  MAC0 Key；0x10   KEK；0x11  MK；0x12   DUKPT IPEK；0x82   Data Key（TPK）
 */
- (void)getSecretKeyInfo:(int)keySystem entype:(int)entype index:(int)index keyType:(int)keyType;

/**
 request Display
 
 @param model 00 Display on the left , 16 Center display
 @param timeout timeout
 @param asciiString  ascii String
 @param buzzerFrequency Buzzer frequency, network byte order, e.g. 1000Hz, the value is 00 00 03 E8
 @param buzzerSoundTime  Buzzer sound time, network byte order, e.g. 1000ms, the value is 00 00 03 E8
 @param buzzerNumber  Buzzer number of times, network byte order, e.g. 3 times, the value is 00 00 00 03
 @param light 00 red light; 01 green light; 02 yellow light; 03 blue light
 @param durationTime duration time ms, network byte order, e.g. 1000ms, the value is 00 00 03 E8
 @param closingTime Closing time ms, network byte order, e.g. 1000ms, the value is 00 00 03 E8
 */
- (void)requestDisplay:(int)model timeout:(int)timeout asciiString:(NSString *)asciiString buzzerFrequency:(int)buzzerFrequency buzzerSoundTime:(int)buzzerSoundTime buzzerNumber:(int)buzzerNumber light:(int)light durationTime:(int)durationTime closingTime:(int)closingTime;

/**
 calculate MAC
 
 * @param entype  1:DUKPT ; 2:MK/SK
 * @param index  index
 * @param mactype  0x00 ECB；0x01 CBC； 0x02 X919； 0x03 XOR ;  0x04 CMAC
 * @param macData macData
*/
- (void)calculateMac:(int)entype index:(int)index mactype:(int)mactype macData:(NSString *)macData;


/**
 download Main Key

 @param index Main Key index
 @param tmk Main key
 */
- (void)downloadMainKey:(int)index tmk:(NSString *)tmk;


/**
 doenload Work key

 @param index main key index
 @param PINkey PINkey
 @param MACkey MACkey
 @param DESkey DESkey
 */
- (void)downloadWorkKey:(int)index PINkey:(NSString *)PINkey MACkey:(NSString *)MACkey DESkey:(NSString *)DESkey;


/**
 download AID Parameters

 @param aIDParameters aIDParameters
 */
- (void)downloadAIDParameters:(AIDParameters *)aIDParameters;


/**
 clear Terminal AID Parameters
 */
- (void)clearAIDParameters;


/**
 download CA Public Key

 @param caPublicKey caPublicKey
 */
- (void)downloadPublicKey:(CAPublicKey  *)caPublicKey;


/**
 clear Terminal Public Key
 */
- (void)clearPublicKey;


/**
 startEmvProcess

 @param timeout Operation timeout
 @param tradeData tradeData
 */
- (void)startEmvProcess:(int)timeout tradeData:(TradeData*)tradeData;



/**
 Send transaction results from the processor back to EMVSwipe
 
 @param data icData
 @param str resultString    00 is success, 01 is  IC card rejected
 */
- (void)sendOnlineProcessResult:(NSString*)data withResultString:(NSString *)str;

/**
 begin NFC/IC

 @param type 0 is IC,1 is NFC
 */
- (void)powerOnAPDU:(int)type timeout:(int)timeout;


/**
 stop NFC/IC
 */
- (void)powerOffAPDU;


/**
 NFC/IC apdu

 @param type 0 is IC,1 is NFC
 @param apduData apdu data
 */
- (void)sendApdu:(int)type apdu:(NSArray*)apduData timeout:(int)timeout;

/**
 Upgrade KEK
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param keyData keydata = key  32bytes
 * @param cmac key 16biytes 0x00 CMAC AES 5bytes
 */
- (void)downloadKEK:(EnType)entype keyLength:(KeyLength)keyLength keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 Upgrade MK
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param index MK index 0-99;
 * @param keyData keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 */
- (void)downloadMK:(EnType)entype keyLength:(KeyLength)keyLength index:(int)index keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 Upgrade DUKPT
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param index DUKPT 1-5
 * @param keyData  keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 * @param ksn DUKPT exists
 */
- (void)downloadDUKPT:(EnType)entype keyLength:(KeyLength)keyLength index:(int)index keyData:(NSString *)keyData cmac:(NSString *)cmac ksn:(NSString *)ksn;

/**
 Upgrade SK
 * @param mainIndex  master key index
 * @param keytype  KeyTypePINkey = 0x01,   KeyTypeDatakey = 0x02,   KeyTypeMackey = 0x03,   KeyTypeKEKkey = 0x10,   KeyTypeMKkey = 0x11,   KeyTypeDUKPTkey = 0x12,   KeyTypeDataTPKkey = 0x82,
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param keyData  keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 */
- (void)downloadSK:(int)mainIndex keytype:(KeyType)keytype entype:(EnType)entype keyLength:(KeyLength)keyLength keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 Upgrade TPK
 * @param mainIndex  master key index
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param keyData  keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 */
- (void)downloadTPK:(int)mainIndex entype:(EnType)entype keyLength:(KeyLength)keyLength keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 get battery power
 */
- (void)getBatteryPower;


/**
 shut down
 */
- (void)shutDown;

/**
 get sdk version
 */
- (NSString *)getSDKVersion;

- (void)startOnlineFirmwareUpdate:(NSString *)url port:(int)port;


- (void)startOnlineKernelUpdate:(NSString *)url port:(int)port;


- (void)startOnlineKeyUpdate:(NSString *)url port:(int)port;


- (void)firmwareUpdateRequest:(DownloadTag*)download;


- (void)receiveData:(NSData *)data;


- (void)firmwareUpdateRequest:(NSString *)type data:(NSData*)data pageNum:(int)pageNum;

@end
