//
//  DebugWindow.m
//
//  Created by itron on 2023/1/12.
//  Copyright © 2023年 Itron. All rights reserved.
//

#import "DebugWindow.h"
#import "LogTool.h"
#import "LogListController.h"

static CGFloat const kSize = 45.0;
#define kFrameWidth [UIScreen mainScreen].bounds.size.width
#define kFrameHeight [UIScreen mainScreen].bounds.size.height

@interface DebugWindow ()

@end

@implementation DebugWindow
+ (instancetype)shareInstance
{
    static DebugWindow *window = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        window = [[DebugWindow alloc]init];
    });
    return window;
}

- (instancetype)init
{
    if(self = [super init]){
        self.frame = CGRectMake(0, 0, kSize, kSize);
        self.center = CGPointMake(kSize * 0.5, kFrameHeight * 0.23);
        self.layer.cornerRadius = kSize * 0.5;
        self.layer.masksToBounds = YES;
        self.windowLevel = UIWindowLevelAlert + 1;
        self.backgroundColor = [[UIColor blackColor]colorWithAlphaComponent:0.2];
        self.rootViewController = [UIViewController new];//必须的
        [self makeKeyAndVisible];
        
        UILabel *label = [[UILabel alloc]initWithFrame:self.bounds];
        label.textColor = [UIColor whiteColor];
        label.font = [UIFont systemFontOfSize:20];
        label.textAlignment = NSTextAlignmentCenter;
        label.text = @"Log";
        [self addSubview:label];
        
        UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tapAction)];
        [self addGestureRecognizer:tapGesture];
        
        UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc]initWithTarget:self action:@selector(panAction:)];
        [self addGestureRecognizer:panGesture];
    }
    return self;
}

+ (void)startDebug
{
    [DebugWindow shareInstance];
    [LogTool logConfig];
}

- (void)tapAction
{
    self.hidden = YES;
    LogListController *logListController = [[LogListController alloc]init];
    UINavigationController *nav = [[UINavigationController alloc] initWithRootViewController:logListController];
    nav.modalPresentationStyle = UIModalPresentationFullScreen;
    UIViewController *rootVc = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    [rootVc presentViewController:nav animated:YES completion:nil];
}

- (void)panAction:(UIPanGestureRecognizer *)panGesture
{
    UIWindow *window = [LogTool p_getCurrentWindow];
    CGPoint panPoint = [panGesture locationInView: window];
    if (panGesture.state == UIGestureRecognizerStateChanged) {
        self.center = CGPointMake(panPoint.x, panPoint.y);
        [panGesture setTranslation:CGPointMake(0, 0) inView:self];
    } else if (panGesture.state == UIGestureRecognizerStateEnded || panGesture.state == UIGestureRecognizerStateCancelled) {
        CGFloat x = 22.5;
        if (panPoint.x > window.bounds.size.width / 2.0) {
            x = window.bounds.size.width - 22.5;
        }
        CGFloat y = MIN(MAX(80, panPoint.y), window.bounds.size.height - 100);
        [panGesture setTranslation:CGPointMake(0, 0) inView:self];
        [UIView animateWithDuration:0.35 animations:^{
            self.center = CGPointMake(x, y);
        }];
    }
}



@end
