//
//  LogListController.m
//
//
//  Created by itron on 2023/1/12.
//  Copyright © 2023年 Itron. All rights reserved.
//

#import "LogListController.h"
#import "LogTool.h"
#import "DebugWindow.h"
#import <QuickLook/QuickLook.h>
#import "LogWebViewController.h"

static NSString *cellIdentifier = @"cell";

@interface YLPreviewItem : NSObject<QLPreviewItem>
- (instancetype)initWithTitle:(NSString *)name;
@end

@implementation YLPreviewItem
@synthesize previewItemTitle = _previewItemTitle;
@synthesize previewItemURL = _previewItemURL;

- (instancetype)initWithTitle:(NSString *)name
{
    if(self = [super init]){
        _previewItemTitle = name;
        NSString *path = [LogTool pathForFile:name];
        _previewItemURL = [NSURL fileURLWithPath:path];
    }
    return self;
}


@end

@interface LogListController ()<QLPreviewControllerDataSource,QLPreviewControllerDelegate>
{
    NSMutableArray *logList;
}
@property(nonatomic,strong)UIDocumentInteractionController *docController;
@property(nonatomic,strong)UIButton *footer;

@end

@implementation LogListController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"All files";
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc]initWithTitle:@"close" style:UIBarButtonItemStylePlain target:self action:@selector(dismiss)];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc]initWithTitle:@"delete all" style:UIBarButtonItemStylePlain target:self action:@selector(clean)];
    [self.tableView registerClass:[UITableViewCell class] forCellReuseIdentifier:cellIdentifier];
    self.tableView.tableFooterView = self.footer;
    logList = [NSMutableArray arrayWithArray:[LogTool allLogFileNames]];
    
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [DebugWindow shareInstance].hidden = NO;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return @"log file";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return logList.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier forIndexPath:indexPath];
    cell.textLabel.font = [UIFont systemFontOfSize:15];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    cell.textLabel.text = logList[indexPath.row];
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [LogTool deleteLogFile:logList[indexPath.row]];
        [logList removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }
}

#pragma mark---UITableViewDelegate
- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return 30;
}

- (NSString *)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return @"delete";
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    LogWebViewController *webController = [[LogWebViewController alloc] initWithFile:logList[indexPath.row]];
    [self.navigationController pushViewController:webController animated:YES];
    
}

#pragma mark---QLPreviewControllerDataSource
- (NSInteger)numberOfPreviewItemsInPreviewController:(QLPreviewController *)controller
{
    return logList.count;
}

- (id<QLPreviewItem>)previewController:(QLPreviewController *)controller previewItemAtIndex:(NSInteger)index
{
    controller.title = logList[index];//注释此行无法正确设置标题。。。
    YLPreviewItem *item = [[YLPreviewItem alloc]initWithTitle:logList[index]];
    return item;
}

#pragma mark---QLPreviewControllerDelegate



#pragma mark---other methods
- (void)dismiss
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)clean
{
    [LogTool deleteAllLogFiles];
    [logList removeAllObjects];
    [self.tableView reloadData];
}



@end
