//
//  LogTool.m
//
//
//  Created by itron on 2023/1/12.
//  Copyright © 2023年 Itron. All rights reserved.
//

#import "LogTool.h"
#import <UIKit/UIKit.h>
#import <unistd.h>
#import "fishhook.h"

static NSString *currentLogFileName;///< 当前日志文件名
static dispatch_queue_t writeLogQueue;


@interface LogTool ()
@end

@implementation LogTool


//函数指针，用来保存原始的函数的地址
static void(*old_nslog)(NSString *format, ...);
 
//新的NSLog
void myNSLog(NSString *format, ...){
     
    va_list vl;
    va_start(vl, format);
    NSString* str = [[NSString alloc] initWithFormat:format arguments:vl];
    va_end(vl);
     
    NSDate *currentDate = [NSDate date];
    NSDateFormatter *dataFormatter = [[NSDateFormatter alloc] init];
    dataFormatter.locale = [NSLocale systemLocale];
    dataFormatter.calendar = [[NSCalendar alloc]initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    [dataFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss.SSSSSSZZ"];
    NSString *dateStr = [dataFormatter stringFromDate:currentDate];
    NSString *logoInfo = [NSString stringWithFormat:@"%@ %@\n",dateStr,str];
    [LogTool log:logoInfo];
    //再调用原来的nslog
    old_nslog(@"%@",str);
}

+ (BOOL)needRedirect
{
    // 判断 当前是否在 模拟器环境 下 在模拟器不保存到文件中
    UIDevice *device = [UIDevice currentDevice];
    if([[device model] hasSuffix:@"Simulator"]){
        return NO;
    }
    return YES;
}

+ (void)logConfig
{
    if(!writeLogQueue){
        writeLogQueue = dispatch_queue_create("com.lambert.log", DISPATCH_QUEUE_SERIAL);//串行队列
    }
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *path = [self logFilePath];
    if(![fileManager fileExistsAtPath:path]){
        [fileManager createFileAtPath:path contents:nil attributes:nil];
    }
    
    //1.保存之前的log
    NSString *initialStr = [NSString stringWithFormat:@"\n"];
    [self log:initialStr];
    
    if(![self needRedirect]) return;
    
    //定义rebinding 结构体
    struct rebinding rebind = {};
    rebind.name = "NSLog";
    rebind.replacement = myNSLog;
    rebind.replaced = (void *)&old_nslog;
    
    //将上面的结构体 放入 reb结构体数组中
    struct rebinding red[]  = {rebind};
    
    /*
     * arg1 : 结构体数据组
     * arg2 : 数组的长度
     */
    
    rebind_symbols(red, 1);
}


+ (NSString *)logDirectory
{
    NSString *docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
    NSString *logDir = [docPath stringByAppendingPathComponent:@"Logs"];
    return logDir;
}

+ (NSString *)logFilePath
{
    if(!currentLogFileName){
        NSString *logDir = [self logDirectory];
        NSFileManager *fileManager = [NSFileManager defaultManager];
        if(![fileManager fileExistsAtPath:logDir]){
            NSError *error;
            [fileManager createDirectoryAtPath:logDir withIntermediateDirectories:YES attributes:nil error:&error];
//            if(error){
//                NSLog(@"%@",error);
//            }
        }
        NSDateFormatter *formatter = [[NSDateFormatter alloc]init];
        [formatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
        NSString *dateStr = [formatter stringFromDate:[NSDate date]];
        NSString *fileName = [dateStr stringByAppendingString:@".txt"];
        currentLogFileName = [logDir stringByAppendingPathComponent:fileName];
    }
    return currentLogFileName;
}

+ (void)log:(NSString *)content
{
    if(!content){
        return;
    }
    
    //这里以追加的形式写入文件
    dispatch_async(writeLogQueue, ^{
        //打开一个文件准备更新（读取或写入）
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForUpdatingAtPath:[self logFilePath]];
        //将文件指针的当前位置（偏移量）放在文件末尾处
        [fileHandle seekToEndOfFile];
        //在文件指针的当前位置写入，写入完成后文件指针的当前位置自动更新
        //采用UTF8编码会导致在浏览器中正常，在真机上显示乱码
        [fileHandle writeData:[content dataUsingEncoding:NSUTF8StringEncoding]];
        [fileHandle closeFile];//关闭文件
    });
}

+ (NSArray *)allLogFileNames
{
    NSMutableArray *fileNames = [NSMutableArray array];
    NSString *logDir = [self logDirectory];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSError *error;
    NSArray *contents = [fileManager contentsOfDirectoryAtPath:logDir error:&error];
//    NSLog(@"%@",error);
    if(contents){
        contents = [contents sortedArrayUsingComparator:^NSComparisonResult(NSString *obj1, NSString *obj2) {
            NSComparisonResult result = [obj1 compare:obj2];
            if(result == NSOrderedAscending) return NSOrderedDescending;
            if(result == NSOrderedDescending) return NSOrderedAscending;
            return NSOrderedSame;
        }];
        [fileNames addObjectsFromArray:contents];
    }
    return fileNames;
}

+ (NSString *)pathForFile:(NSString *)fileName
{
    NSString *logDir = [self logDirectory];
    NSString *filePath = [logDir stringByAppendingPathComponent:fileName];
    return filePath;
}

+ (void)deleteLogFile:(NSString *)fileName
{
    NSString *filePath = [self pathForFile:fileName];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if([fileManager fileExistsAtPath:filePath]){
        NSError *error;
        [fileManager removeItemAtPath:filePath error:&error];
//        NSLog(@"%@",error);
    }
    
}

+ (void)deleteAllLogFiles
{
    NSString *logDir = [self logDirectory];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSError *error;
    [fileManager removeItemAtPath:logDir error:&error];
//    NSLog(@"%@",error);
}

+ (UIWindow *)p_getCurrentWindow {
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (@available(iOS 13.0, *)) {
        for (UIWindowScene *windowScene in [UIApplication sharedApplication].connectedScenes) {
            if (windowScene.activationState == UISceneActivationStateForegroundActive) {
                window = windowScene.windows.firstObject;
                for (int i = 0; i<windowScene.windows.count; i++) {
                    UIWindow *tempWin = [windowScene.windows objectAtIndex:i];
                    if (tempWin.windowLevel == UIWindowLevelNormal) {
                        window = tempWin;
                        break;
                    }
                }
                break;
            }
        }
    }
    if (window == nil || window.windowLevel != UIWindowLevelNormal) {
        for (int i = 0; i<[UIApplication sharedApplication].windows.count; i++) {
            UIWindow *tempWin = [[UIApplication sharedApplication].windows objectAtIndex:i];
            if (tempWin.windowLevel == UIWindowLevelNormal) {
                window = tempWin;
                break;
            }
        }
    }
    return window;
}

+ (UIViewController *)p_getCurrentVC {
    UIWindow * window = [self p_getCurrentWindow];
    UIViewController *result = nil;
    if ([window subviews].count>0) {
        UIView *frontView = [[window subviews] objectAtIndex:0];
        id nextResponder = [frontView nextResponder];
        
        if ([nextResponder isKindOfClass:[UIViewController class]])
            result = nextResponder;
        else
            result = window.rootViewController;
    }
    else{
        result = window.rootViewController;
    }
    if ([result isKindOfClass:[UITabBarController class]]) {
        result = [((UITabBarController*)result) selectedViewController];
    }
    if ([result isKindOfClass:[UINavigationController class]]) {
        result = [((UINavigationController*)result) visibleViewController];
    }
    return result;
}

@end
