//
//  LogWebViewController.m
//
//
//  Created by itron on 2023/1/12.
//  Copyright © 2023年 Itron. All rights reserved.

#import "LogWebViewController.h"
#import "LogTool.h"
#import "DebugWindow.h"
#import <WebKit/WebKit.h>

@interface LogWebViewController ()<WKNavigationDelegate,WKUIDelegate>

@property(nonatomic,copy) NSString *content;

@property (nonatomic, strong) WKWebView *wkWebView;

@property (nonatomic,strong) UIButton *shareButton;

@end

@implementation LogWebViewController

- (instancetype)initWithFile:(NSString *)name
{
    if(self = [super init]){
        self.title = name;
        NSString *path = [LogTool pathForFile:name];
        self.content = path;
    }
    return self;
}

- (WKWebView *)wkWebView {
    if (!_wkWebView) {
        CGFloat height = self.view.frame.size.height;
        
        if ([self isIPhoneXSeries]) {
            CGFloat bottom = 40;
            if (@available(iOS 11.0, *)) {
                UIWindow *mainWindow = [UIApplication sharedApplication].delegate.window;
                if (mainWindow.safeAreaInsets.bottom > 0.0) {
                    bottom = mainWindow.safeAreaInsets.bottom;
                }
            }
            height = self.view.frame.size.height - bottom;
        }
        _wkWebView = [[WKWebView alloc] initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, height)];
        _wkWebView.UIDelegate = self;
        _wkWebView.navigationDelegate = self;
        //自适应
        [_wkWebView sizeToFit];
    }
    return _wkWebView;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.view.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:self.wkWebView];
    UIBarButtonItem *barBnt = [[UIBarButtonItem alloc] initWithCustomView:self.shareButton];
    self.navigationItem.rightBarButtonItem = barBnt;

    NSURL *url = [NSURL fileURLWithPath:self.content];
    [self loadTxt:url];
}

- (BOOL)isIPhoneXSeries{
    BOOL iPhoneXSeries = NO;
    if (UIDevice.currentDevice.userInterfaceIdiom != UIUserInterfaceIdiomPhone) {
        return iPhoneXSeries;
    }
    
    if (@available(iOS 11.0, *)) {
        UIWindow *mainWindow = [UIApplication sharedApplication].delegate.window;
        if (mainWindow.safeAreaInsets.bottom > 0.0) {
            iPhoneXSeries = YES;
        }
    }
    return iPhoneXSeries;
}

- (void)loadTxt:(NSURL *)pageURL {
    NSString *lastName =[[pageURL lastPathComponent] lowercaseString];
    if ([lastName containsString:@".txt"]) {
    //如果为UTF8格式的则body不为空
        NSString *body =[NSString stringWithContentsOfURL:pageURL encoding:NSUTF8StringEncoding error:nil];
 //如果不是 则进行GBK编码再解码一次
        if (!body) {
            body =[NSString stringWithContentsOfURL:pageURL encoding:0x80000632 error:nil];
        }
        //不行用GB18030编码再解码一次
        if (!body) {
            body =[NSString stringWithContentsOfURL:pageURL encoding:0x80000631 error:nil];
        }
        if (body) {
            body =[body stringByReplacingOccurrencesOfString:@"\n" withString:@"<br/>"];//替换换行符为HTML换行符
            [self.wkWebView loadHTMLString:body baseURL:nil];
            return;
        }
        [self.wkWebView loadRequest:[NSURLRequest requestWithURL:pageURL]];
    }else{
        NSURLRequest *request =[NSURLRequest requestWithURL:pageURL];
        [self.wkWebView loadRequest:request];
    }
}

#pragma mark- WKNavigationDelegate delegate
/**
 开始加载web的时候

 @param webView webView description
 @param navigation navigation description
 */
- (void)webView:(WKWebView *)webView didStartProvisionalNavigation:(WKNavigation *)navigation {
    
}

/**
 根据webview 和nanvigationAction的相关信息决定这次跳转是否继续

 @param webView  web描述
 @param navigationAction  navigationAction描述
 @param decisionHandler 请求头的描述
 */
- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    
    decisionHandler(WKNavigationActionPolicyAllow);
    
}

/**
 当网页加载完成的时候调用

 @param webView web描述
 @param navigation 导航的描述
 */
- (void)webView:(WKWebView *)webView didFinishNavigation:(WKNavigation *)navigation {
   
    
}

/**
 加载失败的时候调用

 @param webView webView description
 @param navigation navigation description
 @param error 错误原因
 */
- (void)webView:(WKWebView *)webView didFailProvisionalNavigation:(WKNavigation *)navigation withError:(NSError *)error {
    if (error) {
//        NSLog(@"webView调用失败了---%@",error);
    }
    
}

/**
 跳转失败的时候调用

 @param webView webView description
 @param navigation navigation description
 @param error 错误原因
 */
- (void)webView:(WKWebView *)webView didFailNavigation:(null_unspecified WKNavigation *)navigation withError:(nonnull NSError *)error {
    if (error) {
//        NSLog(@"webView跳转失败了---%@",error);
    }
}

- (UIButton *)shareButton {
    if (!_shareButton) {
        _shareButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_shareButton setTitleColor:[UIColor systemBlueColor] forState:UIControlStateNormal];
        [_shareButton setTitle:@"share" forState:UIControlStateNormal];
        [_shareButton addTarget:self action:@selector(shareBtnClick) forControlEvents:UIControlEventTouchUpInside];
    }
    return _shareButton;
}

- (void)shareBtnClick{

    NSURL *url = [NSURL fileURLWithPath:self.content];
    
    UIActivityViewController *activityVC = [[UIActivityViewController alloc] initWithActivityItems:[NSArray arrayWithObjects:url, nil] applicationActivities:nil];
    if ([[UIDevice currentDevice].model isEqualToString:@"iPad"]) {
        activityVC.modalPresentationStyle = UIModalPresentationPopover;
        activityVC.popoverPresentationController.sourceView = self.shareButton;
        activityVC.popoverPresentationController.sourceRect = self.shareButton.frame;
    }
    activityVC.completionWithItemsHandler = ^(UIActivityType  _Nullable activityType, BOOL completed, NSArray * _Nullable returnedItems, NSError * _Nullable activityError) {
        [DebugWindow shareInstance].hidden = NO;
    };
    
    [DebugWindow shareInstance].hidden = YES;
    [[LogTool p_getCurrentVC] presentViewController:activityVC animated:YES completion:nil];
}

@end
