//
//  ItronBaseClass.h
//  ItronLibrary
//
//  Created by 攀杨 on 2018/8/6.
//  Copyright © 2018年 攀杨. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "CompanyM6PDelegate.h"
#import "CompanyM6PClass.h"
#import "CompanyToolClass.h"
#import "MifareCard.h"

@interface CompanyM6PBluetoothBaseClass : NSObject

+ (CompanyM6PBluetoothBaseClass *)getInstance;
@property (nonatomic,weak)id<CompanyM6PDelegate>delegate;
@property (nonatomic,assign)BOOL debug;



@property (nonatomic,assign)int deviceType;

#pragma Bluetooth

/**
 bluetooth connect state
 */
- (BOOL)isConnected;
/**
 search bluetooth

 @param delegate delegate
 @param timeout timeout
 */
- (void)searchDevices:(id<CompanyM6PDelegate>)delegate timeout:(int)timeout;

/**
 stop search bluetooth
 */
- (void)stopSearching;

/**
 connect bluetooth by UUID

 @param UUIDString UUID of peripheral
 @param cb delegate
 @param timeout timeout
 */
- (void)openDevice:(NSString *)UUIDString cbDelegate:(id<CompanyM6PDelegate>)cb timeout:(int)timeout;

/**
 connect bluetooth by peripheral

 @param peripheral peripheral
 @param cb delegate
 @param timeout timeout
 */
- (void)openDevice:(CBPeripheral *)peripheral delegate:(id<CompanyM6PDelegate>)cb timeout:(int)timeout;




/**
 disconnect bluetooth
 */
-(void)closeDevice;
#pragma mark sendCommand

/**
 request Input
 
 @param showModel 0x00 is displayed on the upper left; 0x10 is automatically centered (only controls the input box)
 @param inputmodel  0x00 Function keys only 0x01 number + Function 0x02 lowercase + Function 0x04 capital + Function 0x08 Symbol + Function 0x0F ALL
 @param minLength  Enter the minimum length, 0 defaults to 0 characters, if skipping is not allowed, please specify a minimum value
 @param maxLength  Enter the maximum length, 0 is 64 characters by default
 @param timeout timeout seconds, 0 defaults to 60s
 @param asciiString The interface title is displayed, automatically left, and the center display needs to manually add spaces
 @param statusBarAscii Interface Status display, automatically to the right
 */
- (void)requestInput:(int)showModel inputmodel:(int)inputmodel minLength:(int)minLength maxLength:(int)maxLength timeout:(int)timeout asciiString:(NSString *)asciiString statusBarAscii:(NSString *)statusBarAscii;


#pragma mark 请求二维码显示
/**
 request QRCode
 
 @param showModel 0x00 is displayed on the upper left; 0x10 is automatically centered (only controls the input box)
 @param asciiString The interface title is displayed, automatically left, and the center display needs to manually add spaces
 @param QRString The interface QRCode is displayed
 @param statusBarAscii Interface Status display, automatically to the right
 @param timeout timeout seconds, 0 defaults to 60s
 */
- (void)requestQRCodeDisplay:(int)showModel asciiString:(NSString *)asciiString QRString:(NSString *)QRString statusBarAscii:(NSString *)statusBarAscii timeout:(int)timeout;

/**
 request InputAndCheck
 @param minLength  Enter the minimum length, 0 is 10 characters by default
 @param maxLength  Enter the maximum length, 0 is 24 characters by default
 @param timeout timeout seconds, 0 defaults to 60s
 */
- (void)requestInputAndCheck:(int)minLength maxLength:(int)maxLength timeout:(int)timeout;

/**
 requestDataEncrypt
 
 @param encryptData encryptData
 @param entype  1:DUKPT ; 2:MK/SK
 @param index  index
 @param random random
 @param dukptMode dukptMode 0 automatic addition of 1 , 1 dupkt  No automatic addition of 1
 @param operationMode operationMode 0 ECB , 1 CBC (IV data is required)
 @param ivData ivData
 */
- (void)requestDataEncrypt:(NSString *)encryptData entype:(int)entype index:(int)index dukptMode:(int)dukptMode random:(NSString *)random operationMode:(int)operationMode ivData:(NSString *)ivData;

/**
 requestDataDecrypt
 
 @param decryptData decryptData
 @param entype  1:DUKPT ; 2:MK/SK
 @param index  index
 @param random random
 @param dukptMode dukptMode 0 automatic addition of 1 , 1 dupkt  No automatic addition of 1
 @param operationMode operationMode 0 ECB , 1 CBC (IV data is required)
 @param ivData ivData
 */
- (void)requestDataDecrypt:(NSString *)decryptData entype:(int)entype index:(int)index dukptMode:(int)dukptMode random:(NSString *)random operationMode:(int)operationMode ivData:(NSString *)ivData;

/**
 getSecretKeyInfo
 
 @param keySystem  1:DUKPT ; 2:MK/SK
 @param entype    0：TDES ,    1：AES
 @param index  index
 @param keyType  0x01  PIN Key；0x02   Data Key；0x03  MAC0 Key；0x10   KEK；0x11  MK；0x12   DUKPT IPEK；0x82   Data Key（TPK）
 */
- (void)getSecretKeyInfo:(int)keySystem entype:(int)entype index:(int)index keyType:(int)keyType;

/**
 request Display
 
 @param model 00 Display on the left , 16 Center display
 @param timeout timeout
 @param asciiString  ascii String
 @param buzzerFrequency Buzzer frequency, network byte order, e.g. 1000Hz, the value is 00 00 03 E8
 @param buzzerSoundTime  Buzzer sound time, network byte order, e.g. 1000ms, the value is 00 00 03 E8
 @param buzzerNumber  Buzzer number of times, network byte order, e.g. 3 times, the value is 00 00 00 03
 @param light 00 red light; 01 green light; 02 yellow light; 03 blue light
 @param durationTime duration time ms, network byte order, e.g. 1000ms, the value is 00 00 03 E8
 @param closingTime Closing time ms, network byte order, e.g. 1000ms, the value is 00 00 03 E8
 */
- (void)requestDisplay:(int)model timeout:(int)timeout asciiString:(NSString *)asciiString buzzerFrequency:(int)buzzerFrequency buzzerSoundTime:(int)buzzerSoundTime buzzerNumber:(int)buzzerNumber light:(int)light durationTime:(int)durationTime closingTime:(int)closingTime;

/**
 calculate MAC
 
 * @param entype  1:DUKPT ; 2:MK/SK
 * @param index  index
 * @param mactype  0x00 ECB；0x01 CBC； 0x02 X919； 0x03 XOR ;  0x04 CMAC
 * @param macData macData
*/
- (void)calculateMac:(int)entype index:(int)index mactype:(int)mactype macData:(NSString *)macData;

/**
 doenload Work key
 
 @param keyType  01:DUKPT 02:MK/SK 03:FixedKey
 @param PINkey PINkey,if not,please nil or @""
 @param MACkey MACkey,if not,please nil or @""
 @param DESkey DESkey,if not,please nil or @""
 */
- (void)downloadWorkKey:(NSString *)keyType PINkey:(NSString *)PINkey MACkey:(NSString *)MACkey DESkey:(NSString *)DESkey;

/**
 pin entry
 @param pin pin
  @param keyType key type
 */
- (void)PINEntry:(NSString *)pin keyType:(NSString *)keyType;

/**
 startEmvProcess
 
 @param timeout Operation timeout
 @param tradeData tradeData
 */
- (void)startEmvProcess:(int)timeout tradeData:(M6PTradeData*)tradeData;

/**
 Send transaction results from the processor back to EMVSwipe
 
 @param data icData
 @param str resultString    00 is success, 01 is  IC card rejected
 */
- (void)sendOnlineProcessResult:(NSString*)data withResultString:(NSString *)str;
/**
 begin NFC/IC
 
 @param type 0 is IC,1 is NFC
 */
- (void)powerOnAPDU:(int)type timeout:(int)timeout;

/**
 stop NFC/IC
 */
- (void)powerOffAPDU;


/**
 NFC/IC apdu
 
 @param type 0 is IC,1 is NFC
 @param apduData apdu data
 */
- (void)sendApdu:(int)type apdu:(NSArray*)apduData timeout:(int)timeout;

/**
 Set terminal automatic shutdown time function
 
 @param time second
 */
- (void)setAutomaticShutdown:(int)time;


/**
 get device information
 */
- (void)getTerminalInfo;

/**
 Stop current operation
 */
- (void)stopTrade;

/**
 Download TerminalInfo
 
 @param TUSN TUSN
  @param bluetoothMac bluetoothMac
 */
- (void)downloadTerminalInfo:(NSString *)TUSN bluetoothMac:(NSString*)bluetoothMac;

/**
 download AID Parameters
 
 @param aIDParameters aIDParameters
 */
- (void)downloadAIDParameters:(M6PAIDParameters *)aIDParameters;


/**
 clear Terminal AID Parameters
 */
- (void)clearAIDParameters;


/**
 download CA Public Key
 
 @param caPublicKey caPublicKey
 */
- (void)downloadPublicKey:(M6PCAPublicKey  *)caPublicKey;


/**
 clear Terminal Public Key
 */
- (void)clearPublicKey;

/**
 Update terminal time function

 @param datetime yyyyMMddHHmmss
 */
- (void)setTerminalDateTime:(NSString *)datetime;

/**
 get terminal time
 */
- (void)getTerminalDateTime;


/**
 get battery power
 */
- (void)getBatteryPower;


/**
 shut down
 */
- (void)shutDown;

/**
 get sdk version
 */
- (NSString *)getSDKVersion;


/**
 Upgrade KEK
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param keyData keydata = key  32bytes
 * @param cmac key 16biytes 0x00 CMAC AES 5bytes
 */
- (void)downloadKEK:(EnType)entype keyLength:(KeyLength)keyLength keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 Upgrade MK
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param index MK index 0-99;
 * @param keyData keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 */
- (void)downloadMK:(EnType)entype keyLength:(KeyLength)keyLength index:(int)index keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 Upgrade DUKPT
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param index DUKPT 1-5
 * @param keyData  keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 * @param ksn DUKPT exists
 */
- (void)downloadDUKPT:(EnType)entype keyLength:(KeyLength)keyLength index:(int)index keyData:(NSString *)keyData cmac:(NSString *)cmac ksn:(NSString *)ksn;

/**
 Upgrade SK
 * @param mainIndex  master key index
 * @param keytype  KeyTypePINkey = 0x01,   KeyTypeDatakey = 0x02,   KeyTypeMackey = 0x03,   KeyTypeKEKkey = 0x10,   KeyTypeMKkey = 0x11,   KeyTypeDUKPTkey = 0x12,   KeyTypeDataTPKkey = 0x82,
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param keyData  keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 */
- (void)downloadSK:(int)mainIndex keytype:(KeyType)keytype entype:(EnType)entype keyLength:(KeyLength)keyLength keyData:(NSString *)keyData cmac:(NSString *)cmac;

/**
 Upgrade TPK
 * @param mainIndex  master key index
 * @param entype    EnTypeTDES = 0,   EnTypeAES = 1,
 * @param keyLength  KeyLength128 = 16,  KeyLength192 = 24,
 * @param keyData  keyData
 * @param cmac key 16biytes 0x00 CMAC   DES 3bytes/ AES 5bytes
 */
- (void)downloadTPK:(int)mainIndex entype:(EnType)entype keyLength:(KeyLength)keyLength keyData:(NSString *)keyData cmac:(NSString *)cmac;

- (void)firmwareUpdateRequest:(NSString *)type data:(NSData*)data pageNum:(int)pageNum;


- (void)receiveData:(NSData *)data;

- (void)updateKey:(NSString*)keyType keyIndex:(NSString*)keyIndex keydata:(NSString *)keydata;

#pragma mark mifare卡实现
- (void)operateMifareClassic:(int)type apdu:(NSArray*)apduData timeout:(int)timeout;

#pragma mark Mifare Classic 扇区读卡数据操作
- (void)readMifareClassic:(MifareCard *)mifareCard withTimeOut:(int)timeout;

#pragma mark Mifare Classic 扇区 写卡数据操作
- (void)writeMifareClassic:(MifareCard *)mifareCard withTimeOut:(int)timeout;

#pragma mark 钱包初始化 初始化后金额为0
- (void)walletInitMifare:(MifareCard *)mifareCard withTimeOut:(int)timeout withMoney:(int)money;

#pragma mark 钱包充值 充值金额 money
- (void)walletRechargeMifare:(MifareCard *)mifareCard withTimeOut:(int)timeout withMoney:(int)money;

#pragma mark 扣款 Deduction
- (void)walletDeductionMifare:(MifareCard *)mifareCard withTimeOut:(int)timeout withMoney:(int)money;

#pragma mark 余额查询
- (void)walletSurplusMifare:(MifareCard *)mifareCard withTimeOut:(int)timeout;

#pragma mark mifare desfire 透传命令
- (void)operateMifareDesfire:(int)type apdu:(NSArray*)apduData timeout:(int)timeout;


@end
