//
//  ItronDelegate.h
//  ItronLibrary
//
//  Created by 攀杨 on 2018/8/6.
//  Copyright © 2018年 攀杨. All rights reserved.
//

#import <CoreBluetooth/CoreBluetooth.h>
#import "CompanyM6PClass.h"
@protocol CompanyM6PDelegate <NSObject>

#pragma mark bluetooth
@optional
/**
 bluetooth search complete
 */
- (void)discoverComplete;

/**
 search bluetooth device
 
 @param peripheral peripheral
 */
- (void)discoverOneDevice:(CBPeripheral *)peripheral;
/**
 error message
 
 @param code 1 blueTooth disconnect 2 BlueToothStatePoweredOff 3 blueTooth receive data error
 @param msg error message
 */
- (void)onError:(NSInteger)code msg:(NSString *)msg;

/**
 if bluetooth is connected
 
 @param isConnect the state of bluetooth connect
 */
- (void)bleConnectState:(BOOL)isConnect;

/**
 command of send timeout
 */
- (void)sendCommandTimeout;

/**
 callback of getTerminal
 
 @param terminalinfo the class of terminalinfo
 @param status the code of terminal return,if 0,is success
 */
- (void)onGetTerminalInfo:(M6PTerminalInfo *)terminalinfo status:(int)status;
/**
 callback of requestData Encrypt or Decrypt
 
 @param encryptOrDecryptData the data of Encrypt or Decrypt
 @param ksn the data of ksn
 @param status the code of terminal return,if 0,is success

 */
- (void)onRequestDataEncrypt:(NSString *)encryptOrDecryptData ksn:(NSString *)ksn status:(int)status;

/**
 callback of requestInput
 
 @param inputString the string of input
 @param status the code of terminal return,if 0,is success

 */
- (void)onRequestInput:(NSString *)inputString status:(int)status;

/**
 callback of requestInput
 
 @param cardNo the cardNo of input
 @param cardexpiryDate the cardexpiryDate of input
 @param status the code of terminal return,if 0,is success

 */
- (void)onRequestInputAndCheck:(NSString *)cardNo cardexpiryDate:(NSString *)cardexpiryDate status:(int)status;

/**
 callback of getSecretKeyInfo
 
 @param keyInfo keyInfo
 @param kcv kcv BYTE0 (1:DUKPT, 2:MK/SK) ；BYTE1(0:TDES,1:AES)；BYTE2(Index)；BYTE3( Key Type)；BYTE4( Key Length)
 @param ksn DUKPT only available ksn
 @param status the code of terminal return,if 0,is success
 */
- (void)onGetSecretKeyInfo:(NSString *)keyInfo kcv:(NSString *)kcv ksn:(NSString *)ksn status:(int)status;

/**
 callback of requestDisplay
 
 @param displayData the data of encrypted
 @param status the code of terminal return,if 0,is success

 */
- (void)onRequestDataDisplay:(NSString *)displayData status:(int)status;

/**
 callback of calculateMac
 
 @param mac data of mac
 @param ksn ksn
 @param status the code of terminal return,if 0,is success
 */
- (void)onCalculateMac:(NSString *)mac ksn:(NSString *)ksn status:(int)status;

/**
 callback of onDownloadKEK
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadKEK:(int)status;

/**
 callback of onDownloadMK or DUKPT
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadMKDUKPT:(int)status;

/**
 callback of onDownloadSK or DUKPT
 @param status the code of terminal return,if 0,is success
 */
-(void)onDownloadSK:(int)status;

/**
 callback of downloadWorkKey
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadWorkKey:(int)status;

/**
 callback of pinEntryRequest
 
 @param pin encry of pin
 @param ksn ksn
 @param status the code of terminal return,if 0,is success
 */
- (void)onPINEntryResult:(NSString *)pin ksn:(NSString *)ksn status:(int)status;

/**
 terminal can swipe
 */
- (void)onWaitingcard;
/**
 IC card have inserted
 */
- (void)onICCardInsertion;

/**
 NFCCard have detected
 */
- (void)onNFCCardDetection;

/**
 Degradation operation waiting for card insertion or swiping
 */
- (void)onWaitingInsertionOrSwipe:(int)code;
/**
 Degradation operation waiting for card swiping
 */
- (void)onWaitingSwipe:(int)code;
/**
 callback of swipe
 
 @param cardInfo the class of card
 @param status the code of terminal return,if 0,is success
 */
- (void)onReadCard:(M6PCardInfo *)cardInfo status:(int)status;


/**
 callback of sendOnlineProcessResult
 
 @param onlineResult onlineRespose result
 @param scriptResult script result
 @param data onlineRespose data
 @param status the code of terminal return,if 0,is success
 */
- (void)onSendOnlineProcessResult:(int)onlineResult scriptResult:(NSString *)scriptResult data:(NSString *)data status:(int)status;

/**
 callback of downloadTerminalInfo
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadTerminalInfo:(int)status;

/**
 callback of setAutomaticShutdown
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onSetAutomaticShutdownStatus:(int)status;

/**
 callback of setTerminalDateTime
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onSetTerminalDateTime:(int)status;


/**
 callback of getTerminalDateTime
 
 @param datetime time of terminal return
 @param status the code of terminal return,if 0,is success
 */
- (void)onGetTerminalDateTime:(NSString *)datetime status:(int)status;


/**
 callback of stopTrade
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onStopTrade:(int)status;

/**
 callback of downloadRSApublicKey
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadRSApublicKey:(int)status;

/**
 callback of setAcceptorTerminalIdentification
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onSetAcceptorTerminalIdentification:(int)status;


/**
 callback of getAcceptorIdentification
 
 @param acceptor acceptor
 @param terminal terminal
 @param status the code of terminal return,if 0,is success
 */
- (void)onGetAcceptorIdentification:(NSString*)acceptor Terminal:(NSString *)terminal status:(int)status;



/**
 callback of getTerminalIdentification
 
 @param terminalIdentification terminalIdentification
 @param status the code of terminal return,if 0,is success
 */
- (void)onGetTerminalIdentification:(NSString*)terminalIdentification status:(int)status;

/**
 callback of downloadMainKey
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadMainKey:(int)status;







/**
 callback of downloadAIDParameters
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadAIDParameters:(int)status;

/**
 callback of clearAIDParameters
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onClearAIDParameters:(int)status;

/**
 callback of downloadPublicKey
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onDownloadPublicKey:(int)status;

/**
 callback of clearPublicKey
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onClearPublicKey:(int)status;





/**
 callback of powerOnAPDU
 
 @param type type
 @param data card data
 @param status the code of terminal return,if 0,is success
 */
- (void)onPowerOnAPDU:(int)type cardData:(NSString *)data status:(int)status;

- (void)onPowerOnAPDU:(NSMutableArray *)data status:(int)status;


/**
 callback of powerOffAPDU
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onPowerOffAPDU:(int)status;


/**
 callback of sendApdu
 
 @param data apdu result
 @param status the code of terminal return,if 0,is success
 */
- (void)onSendApdu:(NSArray *)data status:(int)status;


/**
 callback of getBatteryPower
 
 @param batteryLevel batteryLevel   100%  75%   50%   25%  10%
 @param status the code of terminal return,if 0,is success
 */
- (void)onGetBatteryPower:(NSString *)batteryLevel status:(int)status;


/**
 callback of shutDown
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onShutDown:(int)status;

/**
 callback of closeBluetooth
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onCloseBluetooth:(int)status;

/**
 callback of openBluetooth
 
 @param status the code of terminal return,if 0,is success
 */
- (void)onOpenBluetooth:(int)status;


/**
 get Terminal status  (UART valid)

 @param status  0 Terminal ready
 */
- (void)onGetTerminalStatus:(int)status;

- (void)onFirmwareUpdateRequest:(int)status;

- (void)onFirmwareUpdateRequestProgress:(int)progress;

- (void)onUpdateKey:(int)status;

- (void)testData:(NSString *)str status:(int)status;


- (void)sendBlueData:(NSString*)str;
- (void)receiveBlueData:(NSString *)str;

/**
 callback of cancel
 */
- (void)onCancel;

/**
 callback of sendApdu
 
 @param data apdu result
 @param status the code of terminal return,if 0,is success
 */
- (void)onSendApdu:(NSString*)type data:(NSMutableArray *)data status:(int)status;

@end

